package vertexcover;

import java.util.Arrays;
import pathfinder.JPFMap;
import pathfinder.JPathFinder;
import pathfinder.RandomPathFinder;

/**
 *
 * @author Petr Buryan
 */
public class RandomVCCalculator extends AbstractVCover{
    /**
     * search parameter - probablity of mutation of each array element
     */
    private double m_pMut;

    /**
     * search parameter - number of iterations of the search
     */
    private int m_maxIterations;

    /** Creates the object
     * @param pMut probablity of mutation of each array element
     * @param maxIterations number of iterations
     */
    public RandomVCCalculator(double pMut, int maxIterations){
        m_pMut=pMut;
        m_maxIterations=maxIterations;
    }

    /** Searches minimal number of cities necessary to cover the road graph
     * by means of random mutations
     * @param map the map for the calculations
     * @return IDs of cities selected
     */
    @Override
    public int[] getMinimalVertexCover(JPFMap map) {
        //initialisation stuff
        if(map==null){
            System.err.println("Error, the map data is null!!");
            return null;
        }
        //create the candidate solution
        boolean[]cities=new boolean[map.getCityCount()];
        //initialise the solution = select all cities
        Arrays.fill(cities, true);
        //get the full cover = necessary condition for all solutions
        int roadsTotal=getNumberOfRoadsCovered(cities, map);
        //fitness of the solution=number of cities used. this number is
        //minimised during the run of the algrotihm
        int fitness=cities.length;

        //the loop
        for(int iteration=0;iteration<m_maxIterations;iteration++){
            //create new solution as small change of the current solution
            boolean[]newCities=GAOperators.bMutate(cities, m_pMut);
            int roadsCovered=getNumberOfRoadsCovered(newCities, map);
            if(roadsCovered<roadsTotal)
                continue;//does not cover all roads - it is necessary for
                         //all admissible solutions to cover all the roads
            //check the fitness of new solution
            int newFitness=getNumberOfCitiesUsed(newCities);

            if(newFitness<fitness){
                //the new solution is better than the previous one => select
                //the new one as the best
                fitness=newFitness;
                cities=newCities;
            }
        }
        return booleans2ints(cities);
    }

     /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        //initialise the path finder - RandomPathFinder shall be replaced
        // by student's own implementation
        final JPathFinder pathFinder = new RandomPathFinder();

        //initialise the vertex cover calculator - RandomVCCalculator shall
        // be replaced by student's own implementation
        final RandomVCCalculator vc = new RandomVCCalculator(0.1d, 10000);


        //run the gui
        if(args.length==0)
            pathFinder.showGUI(vc);
        else
            pathFinder.showGUI(vc, args[0]);
    }

}
